#include <QApplication>
#include <QTranslator>
#include <QDebug>
#include <QCommandLineParser>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QDBusError>

#include "mainwindow.h"
#include "ogcommandline.h"
#include "desktopparser.h"
#include "dbusinterface.h"


const QString SERVICE_NAME{"es.opengnsys.OGBrowser.browser"};

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QCoreApplication::setApplicationName("OGBrowser");
    QCoreApplication::setOrganizationName("OpenGnsys");
    QCoreApplication::setOrganizationDomain("opengnsys.es");

    QTranslator translator;
    QStringList translationDirs{QCoreApplication::applicationDirPath(), "", "."};

    DBusInterface dbus_interface = DBusInterface();


    bool translationsOk = false;

    for(const QString &dir : translationDirs) {
        qDebug() << "Trying to find translations in" << dir;
        if (translator.load(QLocale(), "OGBrowser_", "", dir)) {
            qDebug() << "Translations loaded";
            QCoreApplication::installTranslator(&translator);
            translationsOk = true;
            break;
        }
    }

    if (!translationsOk) {
        qWarning() << "Failed to load translations. Tried looking in:" << translationDirs;
    }


    auto &desktopParser = DesktopParser::getInstance();
    desktopParser.loadSchemes();


    QCommandLineParser parser;
    QCommandLineOption openInNewTabOption("openInNewTab", "Create a new tab when opening a new page");
    parser.addOption(openInNewTabOption);

    QCommandLineOption replacePageOption("replacePage", "When trying to open a new page/tab, replace the current page instead");
    parser.addOption(replacePageOption);

    QCommandLineOption ignoreNewPageOption("ignoreNewPage", "Ignore attempts to open documents in a new page/tab");
    parser.addOption(ignoreNewPageOption);

    QCommandLineOption disableWebSecurityOption("disable-web-security", "Disable web security and allow custom URL schemes");
    parser.addOption(disableWebSecurityOption);

    QCommandLineOption noSandboxOption("no-sandbox", "Disable Chromium sandbox to allow running as root");
    parser.addOption(noSandboxOption);

    QCommandLineOption ignoreSslErrors("ignore-ssl-errors", "Ignore SSL certificate errors");
    parser.addOption(ignoreSslErrors);

    QCommandLineOption stopRunning("stop-running", "Stop any running instances");
    parser.addOption(stopRunning);


    QCommandLineOption helpOption  = parser.addHelpOption();

    OGCommandLineOptions &options = OGCommandLineOptions::getInstance();


    parser.process(a);
    QStringList positional = parser.positionalArguments();

    if (parser.isSet(helpOption)) {
        return 0;
    }

    auto connection = QDBusConnection::sessionBus();
    if (!connection.isConnected()) {
        qWarning() << "Can't connect to the D-Bus session bus!";
    } else {
        if (parser.isSet(stopRunning)) {
            QDBusInterface iface(SERVICE_NAME, "/");
            if(iface.isValid()) {
                auto reply = iface.call("quit");
                return 0;
            } else {
                qCritical() << "Can't connect to running browser";
                return 1;
            }

            return 0;
        }


        if (!connection.registerService(SERVICE_NAME)) {
            qCritical() << "Can't register service" << SERVICE_NAME << ", browser already running";

            if (positional.length() > 0) {
                qInfo() << "Setting URL on running browser";
                QDBusInterface iface(SERVICE_NAME, "/");
                if (iface.isValid()) {
                    auto reply = iface.call("setURL", positional[0]);
                    exit(0);
                }
            }
            exit(1);
        }

        qDebug() << "DBus service" << SERVICE_NAME << "registered";

        connection.registerObject("/", &dbus_interface, QDBusConnection::ExportAllSlots);
        qDebug() << "DBus object registered";
    }

    if (parser.isSet(openInNewTabOption)) {
        options.setNewTabBehavior(OGCommandLineOptions::NewTabBehavior::NewTab);
    }

    if (parser.isSet(replacePageOption)) {
        options.setNewTabBehavior(OGCommandLineOptions::NewTabBehavior::ReplacePage);
    }

    if (parser.isSet(ignoreNewPageOption)) {
        options.setNewTabBehavior(OGCommandLineOptions::NewTabBehavior::Ignore);
    }

    if (parser.isSet(ignoreSslErrors)) {
        options.setIgnoreSslErrors(true);
    }


    if ( positional.length() > 0) {
        options.setUrl( positional[0 ] );
    }

    MainWindow w;
    dbus_interface.setMainWindow(&w);
    w.show();
    return a.exec();
}
